/*! \file commentstripstream.hh
    libfsf, generating code to fill structures from configuration files.
    Copyright (C) 2007 Matthieu Nottale. 

    This file is part of libfsfc.

    libfsfc is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License version 3,
    as published by the Free Software Foundation.

    libfsfc is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with libfsfc.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef COMMENTSTRIPSTREAM_HH
#define COMMENTSTRIPSTREAM_HH

#include <iostream>
#include <string>
#include <boost/any.hpp>
#include <fsfc/hash.hh>
#include "equipstreambuf.hh"

class CommentStripStreambuf: public ChainedStreambuf
{
public:
  CommentStripStreambuf(std::streambuf& base): ChainedStreambuf(base),
  wasNewLine(false)
  {
    setEmpty(false);
  }
  boost::any & getOpt(const std::string & name)
  {
    return options[name];
  }
  void setOpt(const std::string &name, const boost::any &t)
  {
    options[name] = t;
  }
protected:
  virtual int underflow()
  {
    int r = baseGet();
    if (r == '\n')
    {
      if (wasNewLine)
	return underflow();
      wasNewLine = true;
    }
    else
      wasNewLine = false;
    if (r == '#')
    {
      //leave the newline
      while (r != EOF && r != '\n')
	r = baseGet();
    }
    if (r == EOF)
      setEmpty(false);
    else
      setAvailable(r, false);
    return r;
  }
  
private:
  libport::hash_map<std::string, boost::any> options;
  bool wasNewLine;
};


inline boost::any& getStreamOption(std::ios& s, const std::string &name)
{
  return ChainedStreambuf::getStreambufByType(s,
    (CommentStripStreambuf*)0, true)->getOpt(name);
}
inline void setStreamOption(std::ios& s, const std::string &name, 
  const boost::any &v)
{
  ChainedStreambuf::getStreambufByType(s,
    (CommentStripStreambuf*)0, true)->setOpt(name,v);
}
#endif
