/*! \file poscountingstream.hh
    libfsf, generating code to fill structures from configuration files.
    Copyright (C) 2007 Matthieu Nottale. 

    This file is part of libfsfc.

    libfsfc is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License version 3,
    as published by the Free Software Foundation.

    libfsfc is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with libfsfc.  If not, see <http://www.gnu.org/licenses/>.
*/
#ifndef POSCOUNTINGSTREAM_HH
#define POSCOUNTINGSTREAM_HH
#include <utility>
#include "equipstreambuf.hh"

using std::pair;


class PosCountingStreambuf: public ChainedStreambuf
{
public:

  PosCountingStreambuf(std::streambuf &s):ChainedStreambuf(s), line(1), col(1)
  {
    setEmpty();
  }
  typedef std::pair<int, int> Position;
  Position getCurrentPos()
  {
    return Position(line, col);
  }
  int getNColsForLine(int tline)
  {
    return nCols[ tline%10];
  }
protected:
  virtual int underflow()
  {
    int c = baseGet();
    if (c != '\n')
      col++;
    else 
    {
      nCols[line%10] = col;
      ++line;
      col = 1;
    }
    setAvailable(c);
    return c;
  }
private:
  int line;
  int col;
  int nCols[10];
};


inline std::pair<int, int> getStreamPosition(std::ios &s)
{
   PosCountingStreambuf * u = ChainedStreambuf::getStreambufByType(s,
    (PosCountingStreambuf*)0, true);
   return u->getCurrentPos();
}

inline int getNColsForLine(std::ios &s, int l)
{
   PosCountingStreambuf * u = ChainedStreambuf::getStreambufByType(s,
    (PosCountingStreambuf*)0, true);
   return u->getNColsForLine(l);
}


#ifdef TEST_POSCOUNTINGSTREAM
#include <sstream>
int main()
{
  PosCountingStreambuf::Position p;
  std::istringstream ss;
  ss.str("coin\npan\nglou\n");
  p = getStreamPosition(ss);
  std::cerr << p.first<<' '<<p.second<<std::endl;
  while (false && ss.good())
  {
    ss.get();
    p = getStreamPosition(ss);
    std::cerr << p.first<<' '<<p.second<<std::endl;
  }
  std::string s;
  ss >>s;
  std::cerr <<s<<" - " <<ss.str()<<std::endl;
  p = getStreamPosition(ss);
  std::cerr << p.first<<' '<<p.second<<std::endl;
  ss >>s;
  std::cerr <<s<<" - " <<ss.str()<<std::endl;
  p = getStreamPosition(ss);
  std::cerr << p.first<<' '<<p.second<<std::endl;
  int a=ss.get(); int b=ss.get();
  p = getStreamPosition(ss);
  std::cerr << p.first<<' '<<p.second<<' '<<a<<' '<<b<<std::endl;
  ss.get(); ss.get();
  p = getStreamPosition(ss);
  std::cerr << p.first<<' '<<p.second<<std::endl;
}
#endif
#endif
