/*! \file trace.hh
    libfsf, generating code to fill structures from configuration files.
    Copyright (C) 2007 Matthieu Nottale. 

    This file is part of libfsfc.

    libfsfc is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License version 3,
    as published by the Free Software Foundation.

    libfsfc is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with libfsfc.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef TRACE_HH
#define TRACE_HH

/*! Simple debug system with priority level and indentation.

*/
#ifndef TRACELEVEL
/// Maximum level below which to trace. Use your CPPFLAGS to override this.
#define TRACELEVEL 0  
#endif


#define TRACE(level, content) do {if (TRACELEVEL > level) \
    std::cerr<<traceLevel(0) << content<<std::endl;} while(0)

/// Trace entering and exiting from current block.
#define TRACE_ME TraceMe _trace_me_(__PRETTY_FUNCTION__)



inline std::string traceLevel(int inc)
{
  static int tLevel = 0;
  tLevel += inc;
  if (inc <0)
    inc = 0;
  std::string s;
  s.reserve(inc);
  s.insert(s.begin(), tLevel, ' ');
  return s;
}

class TraceMe
{
public:
  TraceMe(const char * c) :me(c)
  {
    traceLevel(1);
    TRACE(1,  '+' << me);
    traceLevel(1);
  }
  ~TraceMe()
  {
    traceLevel(-1);
    TRACE(1, '-' << me);
    traceLevel(-1);
  }
  std::string me;
};

#endif
