/*! \file undoablestream.hh
    libfsf, generating code to fill structures from configuration files.
    Copyright (C) 2007 Matthieu Nottale. 

    This file is part of libfsfc.

    libfsfc is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License version 3,
    as published by the Free Software Foundation.

    libfsfc is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with libfsfc.  If not, see <http://www.gnu.org/licenses/>.
*/
#ifndef UNDOABLESTREAM_HH
#define UNDOABLESTREAM_HH

#include <iostream>
#include <string>

#include "fsfc/equipstreambuf.hh"
#ifndef UNDOABLE_DEBUG
#define UNDOABLE_DEBUG 0
#endif

class UndoableStreambuf: public ChainedStreambuf
{
public:
  void dstat()
  {
    if (UNDOABLE_DEBUG)
    std::cerr <<"status: buf '"<<buf<<"' ptrs "<< (void*)eback() <<' '
    << (void *)gptr() <<' ' <<(void *)egptr()<<' '<< 
    ((gptr()==egptr())?-1:((int)*gptr()))<< std::endl;
  }
  UndoableStreambuf(std::streambuf &base): ChainedStreambuf(base)
  {
    setEmpty(false);
    dstat();
  }
  inline const std::string & getUndoBuffer() {return buf;}
  inline void unread(const std::string & undo)
  {
    if (undo.empty())
      return;
    if (UNDOABLE_DEBUG)
    std::cerr <<"unread begin:"<<undo << std::endl;
    dstat();
    buf = undo + buf;
    if (gptr()!=egptr())
    {
      //someone did put back a char without telling us
      // using >>(std::string) ends us like this and there seem to be no way to
      // prevent it
      buf += *gptr();
      setEmpty(false);
      //std::cerr <<"########undoablestream: glitch!"<<std::endl;
    }
    
    if (UNDOABLE_DEBUG)
    std::cerr <<"unread end"<<std::endl;
    dstat();
  }
protected:
  virtual int underflow()
  {
    if (UNDOABLE_DEBUG)
    std::cerr <<"uflw begin"<<std::endl;
    dstat();
    
    int r;
    if (!buf.empty())
    {
      r = buf[0];
      buf = buf.substr(1, buf.npos);
    }
    else
      r = baseGet();
    setAvailable(r, false);
    if (UNDOABLE_DEBUG)
    std::cerr <<"uflw end"<<std::endl;
    dstat();
    return r;
  }
  
virtual int pbackfail(int c = EOF)
{
  std::cerr <<"backfail "<<c<<std::endl;
  if (c == EOF)
    return EOF;
  char cc = static_cast<char>(c);
  std::string s(&cc, &cc + 1);
  unread(s);
  return c;
}
private:
  std::string buf;
};



inline void undoStream(std::istream &s, const std::string & undo)
{
  UndoableStreambuf * u = ChainedStreambuf::getStreambufByType(s,
    (UndoableStreambuf*)0, true);
  u->unread(undo);
}

inline const std::string & undoBuffer(std::istream &s)
{
  UndoableStreambuf * u = ChainedStreambuf::getStreambufByType(s,
    (UndoableStreambuf*)0, true);
  return u->getUndoBuffer();
}

inline void undoStream(std::istream &s, char c)
{
  std::string str;
  str += c;
  undoStream(s, str);
}

#ifdef TEST_UNDOABLESTREAM
#include <sstream>
int main(int argc, const char * argv[])
{
  std::istringstream ss;
  ss.str("coin pan");
  std::string s;
  ss >> s;
  std::cerr <<s<<std::endl;
  s = "rien";
  undoStream(ss, "blop ");
  ss >> s;
  std::cerr <<s<<std::endl;
  undoStream(ss, "mlup plonk ");
  s = "rien";
  ss >> s;
  std::cerr <<s<<std::endl;
  s = "rien";
  ss >> s;
  std::cerr <<s<<std::endl;
  s = "rien";
  ss >> s;
  std::cerr <<s<<std::endl;
  s = "rien";
  ss >> s;
  std::cerr <<s<<std::endl;
  s = "rien";
  ss >> s;
  std::cerr <<s<<std::endl;
  s = "rien";
  ss >> s;
  std::cerr <<s<<std::endl;
}
#endif
#endif
