#! /bin/sh

fatal()
{
  echo "fatal error: $1"
  exit 1
}
absolute ()
{
  local res
  case $1 in
   \\/* | ?:\\/*)
      # Absolute paths don't need to be expanded.
      res=$1
      ;;
   *) local dir=$(pwd)/$(dirname "$1")
      if test -d "$dir"; then
        res=$(cd "$dir" 2>/dev/null && pwd)/$(basename "$1")
      else
        fatal "not a directory: $dir"
      fi
      ;;
  esac

  # On Windows, we must make sure we have a Windows-like UNIX-friendly path (of
  # the form C:/cygwin/path/to/somewhere instead of /path/to/somewhere, notice
  # the use of forward slashes in the Windows path.  Windows *does* understand
  # paths with forward slashes).
  case $(uname -s) in
    CYGWIN*) res=$(cygpath "$res")
  esac
  echo "$res"
}


chk=$1
# ./../../../tests/2.x/andexp-pipeexp.chk -> 2.x
medir=$(basename $(dirname "$chk"))
# ./../../../tests/2.x/andexp-pipeexp.chk -> 2.x/andexp-pipeexp
me=$medir/$(basename "$chk" ".chk")
# ./../../../tests/2.x/andexp-pipeexp.chk -> andexp-pipeexp
meraw=$(basename $me)    # MERAW!


if ! test -d $top_srcdir || test "x$top_srcdir" = x ; then
  echo "top_srcdir not set or not a directory"
  exit 1
fi


chk=$(absolute $chk)
top_srcdir=$(absolute $top_srcdir)
top_builddir=$(absolute $top_builddir)
# Move to a private dir.
rm -rf $me.dir
mkdir -p $me.dir
cd $me.dir


if ! test -f $chk ; then
  echo check file $chk not found
  exit 1
fi

theclass=$(echo $meraw | sed -e 's/-.*//')

#divide the file into code, input and output

input_line=$(grep -n INPUT $chk | cut -d: -f 1)
output_line=$(grep -n OUTPUT $chk |cut -d: -f 1)

tail -n +$(( $output_line + 1 )) $chk >output.exp
tail -n +$(( $input_line + 1 )) $chk | head -n $(( $output_line - $input_line - 1 )) >input.txt
head -n $(( $input_line - 1 )) $chk >code.cc


#invoke fsfcgen
fsfcgen=$top_srcdir/fsfcgen
test -f $fsfcgen

$fsfcgen -o fsfc_code.cc code.cc


#compile it
if ! g++ -o test -I$top_srcdir/include $CPPFLAGS \
     -DTEST_CLASS=$theclass \
     -include code.cc -include fsfc_code.cc  $top_srcdir/tests/test-output.cc \
     -L$top_builddir/src/.libs -lfsfc 2>compile.err; 
  then
  echo "COMPILE FAILED"
  cat compile.err
  exit 1
fi


#run it
./test <input.txt >test.out 2>test.err || true
status=$?
echo $status >test.stat

diff  -u output.exp test.out >output.diff
if test $? != 0;
then
  echo "INPUT:"
  cat $chk
  echo ==================
  echo "DIFF:"
  cat output.diff
  echo ==================
  echo "ERR:"
  cat test.err
exit 1
else
exit 0
fi



