#! /usr/bin/perl -w
#  \file hash.hh
#    libfsf, generating code to fill structures from configuration files.
#    Copyright (C) 2007 Matthieu Nottale.
#
#    This file is part of libfsfc.
#
#    libfsfc is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License version 3,
#    as published by the Free Software Foundation.
#
#    libfsfc is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with libfsfc.  If not, see <http://www.gnu.org/licenses/>.


use warnings;


use XML::XPath;


my %interfaces; 
if (exists $ENV{"INTERFACE"})
{
  my @ifaces = split / /,$ENV{"INTERFACE"};
  for (@ifaces) { $interfaces{$_} = 1 };
}

my %nointerfaces; 
if (exists $ENV{"NOINTERFACE"})
{
  my @ifaces = split / /,$ENV{"NOINTERFACE"};
  for (@ifaces) { $nointerfaces{$_} = 1 };
}

foreach $file (@ARGV) {
#my $parser = XML::DOM::Parser->new();
#my $doc = $parser->parsefile($file);
print '//from file ', $file, "\n";
my $xp = XML::XPath->new(filename => $file);

my $pos = 0;
my $lastname="";
foreach 
my $compound ($xp->find('/doxygen/compounddef[@kind="class"][@prot="public"]')->get_nodelist){
  my $t = $xp->find('templateparamlist', $compound);
  if ($t->size) {
    print '//ignoring template', $xp->find('compoundname', $compound), "\n";
    next;
  }
  
  my $classname = $xp->find('compoundname', $compound)->string_value;
  my $realcname = $classname;
  my $varimpl = '';
  $realcname =~ s/.*:://;
  my $fullname = $classname;
  $fullname =~ s/:://g;
  
  #build number of occurrences by name
  my %nOccurr;
  #hash of ref to catch multiple entries (happens with qt)
  my %uid;
  my $hasVirtual = 0;
  foreach my $func  ($xp->find
  ('./sectiondef[@kind="public-func" or @kind="public-static-func"]/memberdef[@kind="function"][@prot="public"]', $compound)
  ->get_nodelist
  ) {
    my $fname = $xp->find('name', $func)->string_value;
    my $ref = $xp->find('@id', $func)->string_value;
    #print STDERR  "ref: $ref\n";
    if ( ! exists $uid{$ref} ) {
      $nOccurr{$fname}++;
      $uid{$ref} = 0;
    }
    my $virt = $xp->find('@virt', $func)->string_value;
    $hasVirtual = 1 if ($virt eq "virtual" || $virt eq "pure-virtual");    
  }
  
  my $interface = 0;
  $interface = 1 if (($hasVirtual && exists $interfaces{'all'} &&  ! exists $nointerfaces{$realcname}) ||
    (exists $interfaces{$realcname} ));
  
  my $iface_allvirtual = 1;
  my $iface_extra = '';
  if ($interface)
  {
    #seek extra directives in ENV
    if (exists $ENV{'ALL_FORCE'})
    {
      my $ie = $ENV{'ALL_FORCE'};
      $ie =~ s/;/\n/;
      $iface_extra .= $ie;
      $iface_allvirtual = 0;
    }
    my $target = $realcname.'_FORCE';
    while (exists $ENV{$target})
    {
      my $t = $ENV{$target};
      if ($t =~  /^@(.*)$/) {
	$target = $1;
	next;
      }
      $iface_allvirtual = 0;
      $t =~ s/;/\n/;
      $iface_extra .= $t;
      last;
    }
  }
  print '// real name ', $realcname, "\n";
  if ($interface)
  {
    print 'HANDLE_INTERFACE_BEGIN(', $classname, ',',$fullname,')', "\n";
    if ($iface_allvirtual) {
      #handle all virtual methods here
      foreach my $func ($xp->find
	('./sectiondef[@kind="public-func" or @kind="public-static-func"]/memberdef[@kind="function"][@prot="public"][@virt="virtual" or @virt="pure-virtual"]', $compound)
	->get_nodelist
	) 
      {
	my $fname = $xp->find('name', $func)->string_value;
	my $ref = $xp->find('@id', $func)->string_value;
	next if $uid{$ref} > 0;
	$uid{$ref} ++;
	next if $fname =~ /~/;
	
	#get return type and parameters
	my $rettype =  $xp->find('type', $func)->string_value;
	my $params = '';
	my $paramCount = 0;
	foreach my $param ($xp->find("param", $func)->get_nodelist)
	{
	  $params.= ' , ';
	  $paramCount++;
	  $params .= $xp->find("type", $param)->string_value;
	}
	print 'UTryBindInterface', $paramCount, '(',$classname,',',$fname,',';
	print  $rettype, $params, ")\n";
      }
    }
    print $iface_extra;
    print "\n";
    print 'HANDLE_INTERFACE_END_INTERFACE(', $classname, ',',$fullname,')',"\n";
  }
  else 
  {
    my $c1 =  $xp->find('briefdescription', $compound)->string_value;
    my $c2 =  $xp->find('detaileddescription', $compound)->string_value;
    print 'HANDLE_CLASS_BEGIN(', $classname, ',',$fullname,')', "\n";
    if (1 ||   $c1  =~ /[^ \n]/ || $c2=~/[^ \n]/ )
    {
      print 'HANDLE_CLASS_COMMENT((',$c1,'),(', $c2, '))', "\n";
    }
    print "HANDLE_CLASS_BEGIN_BIND($classname, $fullname)\n";
  }

  my $baseclass=$xp->find("./basecompoundref",$compound)->string_value;
  if ($baseclass ne '') {
    my $basefull = $baseclass;
    $basefull =~ s/:://g;
    print "InheritBind($classname, $fullname, $baseclass, $basefull)\n";
  }
  my %vars;
  foreach my $var ($xp->find('./sectiondef[@kind="var" or @kind = "public-attrib"]/memberdef[@kind="variable" and @prot="public"]', $compound)->get_nodelist) {
    my $vname = $xp->find('name', $var)->string_value;
    next if exists $vars{$vname};
    my $vtype = $xp->find('type', $var)->string_value;
    $vars{$vname} = 1;
    my $str = $vtype. ',' . $vname .",$fullname)\n";
    print 'UTryBindVar(',$str;
    my $c1 =  $xp->find('briefdescription', $var)->string_value;
    my $c2 =  $xp->find('detaileddescription', $var)->string_value;
    if (  $c1  =~ /[^ \n]/ || $c2=~/[^ \n]/ ) {
      print
      'HANDLE_VAR_COMMENT(',$vtype,',',$vname,',(',$c1,'),(',$c2,'))',"\n";
    }
    $varimpl .= 'UTryBindVarImpl('.$str;
  }
    
  
  
      
  #pass on all functions
  my $ctor = "";
  my $ctorimpl="";
  my $ctorcnt = 0;
  foreach $func  ($xp->find
  ('./sectiondef[@kind="public-func" or @kind="public-static-func"]/memberdef[@kind="function"][@prot="public"]', $compound)
      ->get_nodelist
      ) {
    my $fname = $xp->find('name', $func)->string_value;
    my $ref = $xp->find('@id', $func)->string_value;
    next if $uid{$ref} > 0;
    $uid{$ref} ++;
    next if $fname =~ /~/;
    
    if ($fname eq $realcname) {
      my $ctname = 'new'.$realcname;
      $ctname .= $ctorcnt if ($ctorcnt != 0);
      
      $ctor.= "UTryBindCtor($classname,$ctname,$fullname)\n";
      my $pcnt = $xp->find('param', $func)->size;
      $ctorimpl .= 'UTryBindCtorImpl'.$pcnt.'('.$classname. ','.$ctname;
      my $params = '';
      foreach my $parm ($xp->find('param', $func)->get_nodelist) {
	$params .= ','.$xp->find('type', $parm)->string_value;
      }
      $params =~ s/\n//g;
      $params =~ s/\s+/ /g;
      $ctorimpl .= $params .")\n";
       
      $ctorcnt++;
    next;
    }
    
    
    if ($xp->find('templateparamlist',$func)->size) {
      print '//ignoring template method ', $fname, "\n"; 
      next;
    }
  
     my $sz = 1;
    if ($lastname ne $fname) {
      $sz = $nOccurr{$fname};
      #print '// ', $fname, ' occurs ',$sz, "\n";
    }
    #print "//$sz\n";
    if ( $sz> 1) {
      $lastname = $fname;
      $count = 1;
    }
    if ($lastname eq $fname) {
      print 'UTryOverloadBind(', $classname,',', $fname, ',';
      print $xp->find('type', $func), ' ';
      print '(';
      print $classname,'::' if ($xp->find('@static', $func)->string_value eq 'no');
      print '*) ', $xp->find('argsstring', $func) , ',';
      print $fname,$count, ")\n";
      $count++;
    }
    else {
      print 'UTryBind(', $classname, ',', $fname, ")\n";
    }  
  }   
    
    print 'HANDLE_CLASS_END_BIND(', $classname, ")\n";
    print "HANDLE_CLASS_BEGIN_CTORS($classname, $fullname)\n";
    
    #constructors cant be bound for interfaces
    print $ctor unless $interface;
    print 'HANDLE_CLASS_END_CTORS(', $classname, ")\n";
    print $varimpl;
    print $ctorimpl unless $interface;
     
      foreach my $tp  ($xp->find
      ('./sectiondef[@kind="typedef"]/memberdef[@kind="typedef"][@prot="public"]', $compound)->get_nodelist ) {
      print "//TYPEDEF(", $classname, ',', $xp->find('name', $tp)->string_value;
	print ")\n";
    }
    if ($interface) {
      print 'HANDLE_INTERFACE_END_IMPL(', $classname, ',', $fullname,")\n";
    }
    else {
      print 'HANDLE_CLASS_END(', $classname, ',', $fullname,")\n";
    }
     #handle enums
     foreach my $en
     ($xp->find('./sectiondef[@kind="public-type" or @kind="enum"]/memberdef[@kind="enum"]',$compound)->get_nodelist) {
       my $ename = $xp->find("name", $en)->string_value;
       print "HANDLE_ENUM(${classname}::$ename)\n";
     }
}
  
  foreach my $ns 
  ($xp->find('/doxygen/compounddef[@kind="namespace"]')->get_nodelist) {
    my $name = $xp->find('compoundname',$ns)->string_value;
    foreach my $td ($xp->find('./sectiondef[@kind="typedef"]/memberdef[@kind="typedef"][@prot="public"]', $ns)->get_nodelist ) {
       print "//TYPEDEF(", $name, ',', $xp->find('name', $td)->string_value;
	 print ")\n";
    }
    
    foreach my $en
     ($xp->find('./sectiondef[@kind="public-type" or @kind="enum"]/memberdef[@kind="enum"]',$ns)->get_nodelist) {
       my $ename = $xp->find("name", $en)->string_value;
       print "HANDLE_ENUM(${name}::$ename)\n";
     }
  }
  
}
